@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -81,7 +83,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
   
   typedef typename T1::elem_type eT;
   
-  if(strip_inv<T1>::do_inv == true)
+  if(strip_inv<T1>::do_inv)
     {
     // replace inv(A)*B with solve(A,B)
     
@@ -97,8 +99,8 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       {
       // if(auxlib::rudimentary_sym_check(A) == false)
       //   {
-      //   if(is_cx<eT>::no )  { arma_debug_warn("inv_sympd(): given matrix is not symmetric"); }
-      //   if(is_cx<eT>::yes)  { arma_debug_warn("inv_sympd(): given matrix is not hermitian"); }
+      //   if(is_cx<eT>::no )  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not symmetric"); }
+      //   if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not hermitian"); }
       //   
       //   out.soft_reset();
       //   arma_stop_runtime_error("matrix multiplication: problem with matrix inverse; suggest to use solve() instead");
@@ -108,8 +110,8 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       
       if( (arma_config::debug) && (auxlib::rudimentary_sym_check(A) == false) )
         {
-        if(is_cx<eT>::no )  { arma_debug_warn("inv_sympd(): given matrix is not symmetric"); }
-        if(is_cx<eT>::yes)  { arma_debug_warn("inv_sympd(): given matrix is not hermitian"); }
+        if(is_cx<eT>::no )  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not symmetric"); }
+        if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not hermitian"); }
         }
       }
     
@@ -120,7 +122,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     
     // TODO: detect sympd via sympd_helper::guess_sympd(A) ?
     
-    #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
+    #if defined(ARMA_OPTIMISE_SYMPD)
       const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, B) : auxlib::solve_square_fast(out, A, B);
     #else
       const bool status = auxlib::solve_square_fast(out, A, B);
@@ -135,7 +137,7 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     return;
     }
   
-  #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
+  #if defined(ARMA_OPTIMISE_SYMPD)
     {
     if(strip_inv<T2>::do_inv_sympd)
       {
@@ -154,8 +156,8 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       
       // if(auxlib::rudimentary_sym_check(B) == false)
       //   {
-      //   if(is_cx<eT>::no )  { arma_debug_warn("inv_sympd(): given matrix is not symmetric"); }
-      //   if(is_cx<eT>::yes)  { arma_debug_warn("inv_sympd(): given matrix is not hermitian"); }
+      //   if(is_cx<eT>::no )  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not symmetric"); }
+      //   if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not hermitian"); }
       //   
       //   out.soft_reset();
       //   arma_stop_runtime_error("matrix multiplication: problem with matrix inverse; suggest to use solve() instead");
@@ -165,8 +167,8 @@ glue_times_redirect2_helper<true>::apply(Mat<typename T1::elem_type>& out, const
       
       if( (arma_config::debug) && (auxlib::rudimentary_sym_check(B) == false) )
         {
-        if(is_cx<eT>::no )  { arma_debug_warn("inv_sympd(): given matrix is not symmetric"); }
-        if(is_cx<eT>::yes)  { arma_debug_warn("inv_sympd(): given matrix is not hermitian"); }
+        if(is_cx<eT>::no )  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not symmetric"); }
+        if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "inv_sympd(): given matrix is not hermitian"); }
         }
       
       arma_debug_assert_mul_size(At.n_cols, At.n_rows, B.n_rows, B.n_cols, "matrix multiplication");
@@ -260,7 +262,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
   
   typedef typename T1::elem_type eT;
   
-  if(strip_inv<T1>::do_inv == true)
+  if(strip_inv<T1>::do_inv)
     {
     // replace inv(A)*B*C with solve(A,B*C);
     
@@ -296,7 +298,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     
     // TODO: detect sympd via sympd_helper::guess_sympd(A) ?
     
-    #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
+    #if defined(ARMA_OPTIMISE_SYMPD)
       const bool status = (strip_inv<T1>::do_inv_sympd) ? auxlib::solve_sympd_fast(out, A, BC) : auxlib::solve_square_fast(out, A, BC);
     #else
       const bool status = auxlib::solve_square_fast(out, A, BC);
@@ -312,7 +314,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     }
   
   
-  if(strip_inv<T2>::do_inv == true)
+  if(strip_inv<T2>::do_inv)
     {
     // replace A*inv(B)*C with A*solve(B,C)
     
@@ -331,7 +333,7 @@ glue_times_redirect3_helper<true>::apply(Mat<typename T1::elem_type>& out, const
     
     Mat<eT> solve_result;
     
-    #if defined(ARMA_OPTIMISE_SOLVE_SYMPD)
+    #if defined(ARMA_OPTIMISE_SYMPD)
       const bool status = (strip_inv<T2>::do_inv_sympd) ? auxlib::solve_sympd_fast(solve_result, B, C) : auxlib::solve_square_fast(solve_result, B, C);
     #else
       const bool status = auxlib::solve_square_fast(solve_result, B, C);
@@ -521,9 +523,9 @@ glue_times::apply(Mat<typename T1::elem_type>& out, const Glue<T1,T2,glue_times>
   {
   arma_extra_debug_sigprint();
   
-  const sword N_mat = 1 + depth_lhs< glue_times, Glue<T1,T2,glue_times> >::num;
+  constexpr uword N_mat = 1 + depth_lhs< glue_times, Glue<T1,T2,glue_times> >::num;
   
-  arma_extra_debug_print(arma_str::format("N_mat = %d") % N_mat);
+  arma_extra_debug_print(arma_str::format("N_mat = %u") % N_mat);
   
   glue_times_redirect<N_mat>::apply(out, X);
   }
@@ -883,7 +885,7 @@ template<typename T1, typename T2>
 arma_hot
 inline
 void
-glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue_times_diag>& X)
+glue_times_diag::apply(Mat<typename T1::elem_type>& actual_out, const Glue<T1, T2, glue_times_diag>& X)
   {
   arma_extra_debug_sigprint();
   
@@ -899,10 +901,10 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
     {
     arma_extra_debug_print("glue_times_diag::apply(): diagmat(A) * B");
     
-    const diagmat_proxy_check<T1_stripped> A(S1.M, out);
+    const diagmat_proxy<T1_stripped> A(S1.M);
     
-    const unwrap_check<T2> tmp(X.B, out);
-    const Mat<eT>& B     = tmp.M;
+    const quasi_unwrap<T2> UB(X.B);
+    const Mat<eT>& B     = UB.M;
     
     const uword A_n_rows = A.n_rows;
     const uword A_n_cols = A.n_cols;
@@ -913,6 +915,13 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
     
     arma_debug_assert_mul_size(A_n_rows, A_n_cols, B_n_rows, B_n_cols, "matrix multiplication");
     
+    const bool is_alias = (A.is_alias(actual_out) || UB.is_alias(actual_out));
+    
+    if(is_alias)  { arma_extra_debug_print("glue_times_diag::apply(): aliasing detected"); }
+    
+    Mat<eT>  tmp;
+    Mat<eT>& out = (is_alias) ? tmp : actual_out;
+    
     out.zeros(A_n_rows, B_n_cols);
     
     for(uword col=0; col < B_n_cols; ++col)
@@ -920,21 +929,20 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
             eT* out_coldata = out.colptr(col);
       const eT*   B_coldata =   B.colptr(col);
       
-      for(uword i=0; i < A_length; ++i)
-        {
-        out_coldata[i] = A[i] * B_coldata[i];
-        }
+      for(uword i=0; i < A_length; ++i)  { out_coldata[i] = A[i] * B_coldata[i]; }
       }
+    
+    if(is_alias)  { actual_out.steal_mem(tmp); }
     }
   else
   if( (strip_diagmat<T1>::do_diagmat == false) && (strip_diagmat<T2>::do_diagmat == true) )
     {
     arma_extra_debug_print("glue_times_diag::apply(): A * diagmat(B)");
     
-    const unwrap_check<T1> tmp(X.A, out);
-    const Mat<eT>& A     = tmp.M;
+    const quasi_unwrap<T1> UA(X.A);
+    const Mat<eT>& A     = UA.M;
     
-    const diagmat_proxy_check<T2_stripped> B(S2.M, out);
+    const diagmat_proxy<T2_stripped> B(S2.M);
     
     const uword A_n_rows = A.n_rows;
     const uword A_n_cols = A.n_cols;
@@ -945,6 +953,13 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
     
     arma_debug_assert_mul_size(A_n_rows, A_n_cols, B_n_rows, B_n_cols, "matrix multiplication");
     
+    const bool is_alias = (UA.is_alias(actual_out) || B.is_alias(actual_out));
+    
+    if(is_alias)  { arma_extra_debug_print("glue_times_diag::apply(): aliasing detected"); }
+    
+    Mat<eT>  tmp;
+    Mat<eT>& out = (is_alias) ? tmp : actual_out;
+    
     out.zeros(A_n_rows, B_n_cols);
     
     for(uword col=0; col < B_length; ++col)
@@ -954,22 +969,28 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
             eT* out_coldata = out.colptr(col);
       const eT*   A_coldata =   A.colptr(col);
       
-      for(uword i=0; i < A_n_rows; ++i)
-        {
-        out_coldata[i] = A_coldata[i] * val;
-        }
+      for(uword i=0; i < A_n_rows; ++i)  { out_coldata[i] = A_coldata[i] * val; }
       }
+    
+    if(is_alias)  { actual_out.steal_mem(tmp); }
     }
   else
   if( (strip_diagmat<T1>::do_diagmat == true) && (strip_diagmat<T2>::do_diagmat == true) )
     {
     arma_extra_debug_print("glue_times_diag::apply(): diagmat(A) * diagmat(B)");
     
-    const diagmat_proxy_check<T1_stripped> A(S1.M, out);
-    const diagmat_proxy_check<T2_stripped> B(S2.M, out);
+    const diagmat_proxy<T1_stripped> A(S1.M);
+    const diagmat_proxy<T2_stripped> B(S2.M);
     
     arma_debug_assert_mul_size(A.n_rows, A.n_cols, B.n_rows, B.n_cols, "matrix multiplication");
     
+    const bool is_alias = (A.is_alias(actual_out) || B.is_alias(actual_out));
+    
+    if(is_alias)  { arma_extra_debug_print("glue_times_diag::apply(): aliasing detected"); }
+    
+    Mat<eT>  tmp;
+    Mat<eT>& out = (is_alias) ? tmp : actual_out;
+    
     out.zeros(A.n_rows, B.n_cols);
     
     const uword A_length = (std::min)(A.n_rows, A.n_cols);
@@ -977,10 +998,9 @@ glue_times_diag::apply(Mat<typename T1::elem_type>& out, const Glue<T1, T2, glue
     
     const uword N = (std::min)(A_length, B_length);
     
-    for(uword i=0; i < N; ++i)
-      {
-      out.at(i,i) = A[i] * B[i];
-      }
+    for(uword i=0; i < N; ++i)  { out.at(i,i) = A[i] * B[i]; }
+    
+    if(is_alias)  { actual_out.steal_mem(tmp); }
     }
   }
 
