@@ -1,11 +1,19 @@
-// Copyright (C) 2015 National ICT Australia (NICTA)
+// SPDX-License-Identifier: Apache-2.0
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
+// 
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup op_find_unique
@@ -27,11 +35,11 @@ op_find_unique::apply_helper(Mat<uword>& out, const Proxy<T1>& P, const bool asc
   if(n_elem == 0)  { out.set_size(0,1);             return true; }
   if(n_elem == 1)  { out.set_size(1,1); out[0] = 0; return true; }
   
-  uvec indices(n_elem);
+  uvec indices(n_elem, arma_nozeros_indicator());
   
   std::vector< arma_find_unique_packet<eT> > packet_vec(n_elem);
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     typename Proxy<T1>::ea_type Pea = P.get_ea();
     
